#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Easily configure logging for a script
"""

import inspect
import logging
import sys


def configure_logging(args, default=logging.WARNING):
    """Set logging levels based on defaults and command-line arguments

    Keyword arguments:
    args -- an argparse.Namespace object that may contain one or more of the
            following arguments:
                loglevel -- a severity/importance level from "logging"
                verbose -- True = logging.INFO is the desired level
                veryverbose -- True = logging.DEBUG is the desired level
    default -- a severity/importance level from the logging module that is to
               be used as the default logging level; it may be overridden by
               values provided in "args."
    """
    if args.loglevel is not 'NOTSET':
        args_log_level = ''.join(args.loglevel.upper().split())
        try:
            log_level = getattr(logging, args_log_level)
        except AttributeError:
            logging.error(
                'command line option to set log_level failed '
                'because {} is not a valid level name; using {}'
                ''.format(args_log_level, logging.getLevelName(default)))
            log_level = default
    elif args.veryverbose:
        log_level = logging.DEBUG
    elif args.verbose:
        log_level = logging.INFO
    else:
        log_level = default
    logging.basicConfig(level=log_level)
    if log_level != default:
        logging.info(
            'logging level changed to {} via command line option; was {}'
            ''.format(
                logging.getLevelName(log_level),
                logging.getLevelName(default)))
    else:
        logging.info(
            'using default logging level: {}'
            ''.format(logging.getLevelName(default)))


def flog(var, level=logging.DEBUG, comment=None):
    """Easily log a variable's name and value

    Keyword arguments:
    var -- the variable to log
    level -- the desired severity/importance level from the logging module
    comment -- an optional comment to postfix to the logged output
    """
    logger = logging.getLogger(sys._getframe(1).f_code.co_name)
    callers_local_vars = inspect.currentframe().f_back.f_locals.items()
    names = [
        var_name for var_name, var_val in callers_local_vars if var_val is var]
    if len(names) > 0:
        msg = '{}: {}'.format(names[0], repr(var))
    else:
        msg = ' {}'.format(repr(var))
    if comment is not None:
        msg = ''.join((comment, msg))
    logger.log(level, msg)




from distutils.core import setup
from codecs import open
import airtight
from os import path

here = path.abspath(path.dirname(__file__))
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()


setup(
    name='airtight',
    version='0.1',
    description='Components for scripts in python 3.6+',
    long_description=long_description,
    url='http://www.paregorios.org/airtight/',
    author='Tom Elliott',
    author_email='tom.elliott@nyu.edu',
    license='LICENSE.txt',
    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Affero General Public License v3',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
    keywords='scripting',

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=['airtight']

    # Alternatively, if you want to distribute just a my_module.py, uncomment
    # this:
    #   py_modules=["my_module"],

    # List run-time dependencies here.  These will be installed by pip when
    # your project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    # install_requires=['peppercorn'],

    # List additional groups of dependencies here (e.g. development
    # dependencies). You can install these using the following syntax,
    # for example:
    # $ pip install -e .[dev,test]
    # extras_require={
    #     'dev': ['check-manifest'],
    #     'test': ['coverage'],
    # },

    # If there are data files included in your packages that need to be
    # installed, specify them here.  If using Python 2.6 or less, then these
    # have to be included in MANIFEST.in as well.
    # package_data={
    #     'sample': ['package_data.dat'],
    # },

    # Although 'package_data' is the preferred approach, in some case you may
    # need to place data files outside of your packages. See:
    # http://docs.python.org/3.4/distutils/setupscript.html#installing-additional-files # noqa
    # In this case, 'data_file' will be installed into '<sys.prefix>/my_data'
    # data_files=[('my_data', ['data/data_file'])],

    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # pip to create the appropriate form of executable for the target platform.
    # entry_points={
    #     'console_scripts': [
    #         'sample=sample:main',
    #     ],
    # },
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Easily initialize logging level and command-line arguments for a script.
"""
from airtight.logging import configure_logging
import argparse
import inspect
import logging
import sys


def configure_commandline(
        optional_arguments, positional_arguments, default_log_level):
    """Define and parse command line arguments

    Keyword arguments:
    optional_arguments -- a list of lists; each internal list defines an
        optional argument:
            short name (str)
            long name (str)
            default value,
            help string (str),
            required? (boolean)
    positional_arguments -- a list of lists; each internal list defines a
        positional argument:
            name (str)
            type (python type like "str", "int")
            help string (str)
    default_log_level -- an importance/severity level as defined by the Python
        "logging" module (e.g., logging.DEBUG, logging.INFO)
    """
    frame = inspect.currentframe().f_back
    module = inspect.getmodule(frame)
    parser = argparse.ArgumentParser(
        description=inspect.getdoc(module),
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    for a in optional_arguments:
        d = {
            'help': a[3],
            'required': a[4]
        }
        if type(a[2]) == bool:
            if a[2] is False:
                d['action'] = 'store_true'
                d['default'] = False
            else:
                d['action'] = 'store_false'
                d['default'] = True
        else:
            d['type'] = type(a[2])
            d['default'] = a[2]
        parser.add_argument(
            a[0],
            a[1],
            **d)
    for a in positional_arguments:
        d = {
            'type': a[1],
            'help': a[2]
        }
        parser.add_argument(a[0], **d)
    args = parser.parse_args()
    configure_logging(args, default_log_level)
    logging.debug('command line: {}'.format(' '.join(sys.argv)))
    return vars(args)
